@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, ExistentialQuantification #-}
+{-# LANGUAGE ExistentialQuantification #-}
 -- |
 -- Module      : System.ZMQ
 -- Copyright   : (c) 2010-2011 Toralf Wittner
@@ -29,29 +29,19 @@ module System.ZMQ (
   , Pair(..)
   , Pub(..)
   , Sub(..)
-  , XPub(..)
-  , XSub(..)
   , Req(..)
   , Rep(..)
   , XReq(..)
   , XRep(..)
   , Pull(..)
   , Push(..)
-
-#ifdef ZMQ2
   , Up(..)
   , Down(..)
-#endif
 
   , withContext
   , withSocket
   , setOption
   , getOption
-
-#ifdef ZMQ3
-  , getMsgOption
-#endif
-
   , System.ZMQ.subscribe
   , System.ZMQ.unsubscribe
   , bind
@@ -61,7 +51,7 @@ module System.ZMQ (
   , receive
   , moreToReceive
   , poll
-  , System.ZMQ.zmqVersion
+  , version
 
     -- * Low-level functions
   , init
@@ -69,10 +59,8 @@ module System.ZMQ (
   , socket
   , close
 
-#ifdef ZMQ2
   , Device(..)
   , device
-#endif
 
 ) where
 
@@ -81,7 +69,7 @@ import Control.Applicative
 import Control.Exception
 import Control.Monad (unless, when)
 import Data.IORef (atomicModifyIORef)
-import Foreign hiding (with)
+import Foreign
 import Foreign.C.Error
 import Foreign.C.String
 import Foreign.C.Types (CInt, CShort)
@@ -121,23 +109,6 @@ data Sub = Sub
 instance SType Sub where
     zmqSocketType = const sub
 
--- | Same as 'Pub' except that you can receive subscriptions from the
--- peers in form of incoming messages. Subscription message is a byte 1
--- (for subscriptions) or byte 0 (for unsubscriptions) followed by the
--- subscription body.
--- /Compatible peer sockets/: 'Sub', 'XSub'.
-data XPub = XPub
-instance SType XPub where
-    zmqSocketType = const xpub
-
--- | Same as 'Sub' except that you subscribe by sending subscription
--- messages to the socket. Subscription message is a byte 1 (for subscriptions)
--- or byte 0 (for unsubscriptions) followed by the subscription body.
--- /Compatible peer sockets/: 'Pub', 'XPub'.
-data XSub = XSub
-instance SType XSub where
-    zmqSocketType = const xsub
-
 -- | Socket to send requests and receive replies. Requests are
 -- load-balanced among all the peers. This socket type allows only an
 -- alternated sequence of send's and recv's.
@@ -160,7 +131,7 @@ instance SType Rep where
 -- Replies received by this socket are tagged with a proper postfix
 -- that can be use to route the reply back to the original requester.
 -- /Compatible peer sockets/: 'Rep', 'Xrep'.
-data XReq = Xreq
+data XReq = XReq
 instance SType XReq where
     zmqSocketType = const xrequest
 
@@ -170,7 +141,7 @@ instance SType XReq where
 -- sending a reply via XREP socket the message should be tagged with a
 -- prefix from a corresponding request.
 -- /Compatible peer sockets/: 'Req', 'Xreq'.
-data XRep = Xrep
+data XRep = XRep
 instance SType XRep where
     zmqSocketType = const xresponse
 
@@ -196,7 +167,6 @@ data Push = Push
 instance SType Push where
     zmqSocketType = const push
 
-#ifdef ZMQ2
 {-# DEPRECATED Up "Use Pull instead." #-}
 -- | Socket to receive messages from up the stream. Messages are
 -- fair-queued from among all the connected peers. Send function is not
@@ -212,51 +182,33 @@ instance SType Up where
 data Down = Down
 instance SType Down where
     zmqSocketType = const downstream
-#endif
 
 -- | Subscribable.
 class SubsType a
 
 instance SubsType Sub
-instance SubsType XSub
 
 -- | The option to set on 0MQ sockets (cf. zmq_setsockopt and zmq_getsockopt
 -- manpages for details).
 data SocketOption =
     Affinity        Word64    -- ^ ZMQ_AFFINITY
-  | Backlog         Int       -- ^ ZMQ_BACKLOG
+  | Backlog         CInt      -- ^ ZMQ_BACKLOG
   | Events          PollEvent -- ^ ZMQ_EVENTS
-  | FD              Int       -- ^ ZMQ_FD
+  | FD              CInt      -- ^ ZMQ_FD
   | Identity        String    -- ^ ZMQ_IDENTITY
-  | Linger          Int       -- ^ ZMQ_LINGER
+  | Linger          CInt      -- ^ ZMQ_LINGER
   | Rate            Int64     -- ^ ZMQ_RATE
   | ReceiveBuf      Word64    -- ^ ZMQ_RCVBUF
   | ReceiveMore     Bool      -- ^ ZMQ_RCVMORE
-  | ReconnectIVL    Int       -- ^ ZMQ_RECONNECT_IVL
-  | ReconnectIVLMax Int       -- ^ ZMQ_RECONNECT_IVL_MAX
+  | ReconnectIVL    CInt      -- ^ ZMQ_RECONNECT_IVL
+  | ReconnectIVLMax CInt      -- ^ ZMQ_RECONNECT_IVL_MAX
   | RecoveryIVL     Int64     -- ^ ZMQ_RECOVERY_IVL
   | SendBuf         Word64    -- ^ ZMQ_SNDBUF
-#ifdef ZMQ2
   | HighWM          Word64    -- ^ ZMQ_HWM
   | McastLoop       Bool      -- ^ ZMQ_MCAST_LOOP
   | RecoveryIVLMsec Int64     -- ^ ZMQ_RECOVERY_IVL_MSEC
   | Swap            Int64     -- ^ ZMQ_SWAP
-#endif
-#ifdef ZMQ3
-  | IPv4Only        Bool      -- ^ ZMQ_IPV4ONLY
-  | MaxMsgSize      Int64     -- ^ ZMQ_MAXMSGSIZE
-  | McastHops       Int       -- ^ ZMQ_MULTICAST_HOPS
-  | ReceiveHighWM   Int       -- ^ ZMQ_RCVHWM
-  | ReceiveTimeout  Int       -- ^ ZMQ_RCVTIMEO
-  | SendHighWM      Int       -- ^ ZMQ_SNDHWM
-  | SendTimeout     Int       -- ^ ZMQ_SNDTIMEO
-#endif
-  deriving (Eq, Ord, Show)
-
-#ifdef ZMQ3
-data MessageOption = MoreMsgParts CInt -- ^ ZMQ_MORE
-  deriving (Eq, Ord, Show)
-#endif
+  deriving (Eq, Ord, Show, Read)
 
 -- | The events to wait for in poll (cf. man zmq_poll)
 data PollEvent =
@@ -265,7 +217,7 @@ data PollEvent =
   | InOut  -- ^ ZMQ_POLLIN | ZMQ_POLLOUT
   | Native -- ^ ZMQ_POLLERR
   | None
-  deriving (Eq, Ord, Show)
+  deriving (Eq, Ord, Show, Read)
 
 -- | Type representing a descriptor, poll is waiting for
 -- (either a 0MQ socket or a file descriptor) plus the type
@@ -294,21 +246,10 @@ setOption s (ReconnectIVL o)    = setIntOpt s reconnectIVL o
 setOption s (ReconnectIVLMax o) = setIntOpt s reconnectIVLMax o
 setOption s (RecoveryIVL o)     = setIntOpt s recoveryIVL o
 setOption s (SendBuf o)         = setIntOpt s sendBuf o
-#ifdef ZMQ2
 setOption s (HighWM o)          = setIntOpt s highWM o
 setOption s (McastLoop o)       = setBoolOpt s mcastLoop o
 setOption s (RecoveryIVLMsec o) = setIntOpt s recoveryIVLMsec o
 setOption s (Swap o)            = setIntOpt s swap o
-#endif
-#ifdef ZMQ3
-setOption s (IPv4Only o)        = setBoolOpt s ipv4Only o
-setOption s (MaxMsgSize o)      = setIntOpt s maxMessageSize o
-setOption s (McastHops o)       = setIntOpt s mcastHops o
-setOption s (ReceiveHighWM o)   = setIntOpt s receiveHighWM o
-setOption s (ReceiveTimeout o)  = setIntOpt s receiveTimeout o
-setOption s (SendHighWM o)      = setIntOpt s sendHighWM o
-setOption s (SendTimeout o)     = setIntOpt s sendTimeout o
-#endif
 
 -- | Get the given socket option by passing in some dummy value of
 -- that option. The actual value will be returned. Please note that
@@ -328,27 +269,20 @@ getOption s (ReconnectIVL _)    = ReconnectIVL <$> getIntOpt s reconnectIVL
 getOption s (ReconnectIVLMax _) = ReconnectIVLMax <$> getIntOpt s reconnectIVLMax
 getOption s (RecoveryIVL _)     = RecoveryIVL <$> getIntOpt s recoveryIVL
 getOption s (SendBuf _)         = SendBuf <$> getIntOpt s sendBuf
-#ifdef ZMQ2
 getOption s (HighWM _)          = HighWM <$> getIntOpt s highWM
 getOption s (McastLoop _)       = McastLoop <$> getBoolOpt s mcastLoop
 getOption s (RecoveryIVLMsec _) = RecoveryIVLMsec <$> getIntOpt s recoveryIVLMsec
 getOption s (Swap _)            = Swap <$> getIntOpt s swap
-#endif
-#ifdef ZMQ3
-getOption s (IPv4Only _)        = IPv4Only <$> getBoolOpt s ipv4Only
-getOption s (MaxMsgSize _)      = MaxMsgSize <$> getIntOpt s maxMessageSize
-getOption s (McastHops _)       = McastHops <$> getIntOpt s mcastHops
-getOption s (ReceiveHighWM _)   = ReceiveHighWM <$> getIntOpt s receiveHighWM
-getOption s (ReceiveTimeout _)  = ReceiveTimeout <$> getIntOpt s receiveTimeout
-getOption s (SendHighWM _)      = SendHighWM <$> getIntOpt s sendHighWM
-getOption s (SendTimeout _)     = SendTimeout <$> getIntOpt s sendTimeout
 
-getMsgOption :: Message -> MessageOption -> IO MessageOption
-getMsgOption m (MoreMsgParts _) = MoreMsgParts <$> getIntMsgOpt m more
-#endif
-
-zmqVersion :: (Int, Int, Int)
-zmqVersion = B.zmqVersion
+version :: IO (Int, Int, Int)
+version =
+    with 0 $ \major_ptr ->
+    with 0 $ \minor_ptr ->
+    with 0 $ \patch_ptr ->
+        c_zmq_version major_ptr minor_ptr patch_ptr >>
+        tupleUp <$> peek major_ptr <*> peek minor_ptr <*> peek patch_ptr
+  where
+    tupleUp a b c = (fromIntegral a, fromIntegral b, fromIntegral c)
 
 -- | Initialize a 0MQ context (cf. zmq_init for details).  You should
 -- normally prefer to use 'with' instead.
@@ -360,7 +294,7 @@ init ioThreads = do
 -- | Terminate a 0MQ context (cf. zmq_term).  You should normally
 -- prefer to use 'with' instead.
 term :: Context -> IO ()
-term = throwErrnoIfMinus1_ "term" . c_zmq_term . ctx
+term = throwErrnoIfMinus1Retry_ "term" . c_zmq_term . ctx
 
 -- | Run an action with a 0MQ context.  The 'Context' supplied to your
 -- action will /not/ be valid after the action either returns or
@@ -368,7 +302,7 @@ term = throwErrnoIfMinus1_ "term" . c_zmq_term . ctx
 withContext :: Size -> (Context -> IO a) -> IO a
 withContext ioThreads act =
   bracket (throwErrnoIfNull "c_zmq_init" $ c_zmq_init (fromIntegral ioThreads))
-          (throwErrnoIfMinus1_ "c_zmq_term" . c_zmq_term)
+          (throwErrnoIfMinus1Retry_ "c_zmq_term" . c_zmq_term)
           (act . Context)
 
 -- | Run an action with a 0MQ socket. The socket will be closed after running
@@ -503,7 +437,6 @@ waitRead, waitWrite :: Socket a -> IO ()
 waitRead = wait' threadWaitRead pollIn
 waitWrite = wait' threadWaitWrite pollOut
 
-#ifdef ZMQ2
 -- | Type representing ZeroMQ devices, as used with zmq_device
 data Device =
     Streamer  -- ^ ZMQ_STREAMER
@@ -525,4 +458,4 @@ device device' insock outsock =
     fromDevice Streamer  = fromIntegral . deviceType $ deviceStreamer
     fromDevice Forwarder = fromIntegral . deviceType $ deviceForwarder
     fromDevice Queue     = fromIntegral . deviceType $ deviceQueue
-#endif
+
